//
// THIS FILE IS COPYRIGHT 1994, 1995 BY TELENOR RESEARCH, NORWAY
//
//                        ALL RIGHTS RESERVED.
//
// For rights granted to the public see the COPYRIGHT file
//
//
// SetNode.h	SetNode class for the set expression tree
//
// Author: Haiyan Yang
//
// Changes: January 1994 Brd Hfjeld
//				Added support for search_params to evaluate()
//

#ifndef _SETNODE_H
#define _SETNODE_H

#include <iostream.h>
#include <UP.SplaySet.h>
#include <Disk.h>
#include <String.h>
#include <stdlib.h>
#include <lang.h>

#define URLSet	 URLPTRSplaySet


class SetNode
{
public:
    friend ostream& operator<<(ostream& os, SetNode& n);
    virtual ostream& print(ostream&) = 0;
    virtual URLSet evaluate(Disk&, struct search_params *) = 0;
    virtual int isNot() = 0;
};

// INVALID node to evaulate, used during building of set syntax tree
class NotSetNode : public SetNode
{
private:
    SetNode*	_child;

public:
    NotSetNode(SetNode* child);
    SetNode* child();
    URLSet evaluate(Disk&, struct search_params *);
    ostream& print(ostream& os);
    int isNot();
};


class UnionSetNode : public SetNode
{
private:
    SetNode*	_left;
    SetNode*	_right;

public:
    UnionSetNode(SetNode* left, SetNode* right);
    URLSet evaluate(Disk& d, struct search_params *);
    int isNot();
    ostream& print(ostream& os);
};


class CutSetNode : public SetNode
{
private:
    SetNode*	_left;
    SetNode*	_right;

public:
    CutSetNode(SetNode* left, SetNode* right);
    URLSet evaluate(Disk& d, struct search_params *);
    int isNot();
    ostream& print(ostream& os);
};

class DiffSetNode : public SetNode
{
private:
    SetNode*	_left;
    SetNode*	_right;

public:
    DiffSetNode(SetNode* left, SetNode* right);
    URLSet evaluate(Disk& d, struct search_params *);
    int isNot();
    ostream& print(ostream& os);

};

class TextSetNode : public SetNode
{
private:	
    unsigned char * _str;
public:
    TextSetNode(unsigned char *str);
    URLSet evaluate(Disk& d, struct search_params *);
    int isNot();
    ostream& print(ostream& os);
};


// The inlines
inline ostream& operator<<(ostream& os, SetNode& n)
{
    return n.print(os);
}


// INVALID node to evaulate, used during building of set syntax tree
inline NotSetNode::NotSetNode(SetNode* child)
{
    _child = child;
}

inline SetNode* NotSetNode::child() 
{ 
    return _child; 
}

inline URLSet NotSetNode::evaluate(Disk&, struct search_params *sp)
{
    cerr << "INTERNAL ERROR: NotSetNode should never be evauluated!"
	<< endl;
    exit(1);
}

inline ostream& NotSetNode::print(ostream& os)
{ 
    return os << "!(" << *_child << ")"; 
}

inline int NotSetNode::isNot() 
{ 
    return 1; 
}


inline UnionSetNode::UnionSetNode(SetNode* left, SetNode* right)
{
    _left = left; 
    _right = right;
}

inline URLSet UnionSetNode::evaluate(Disk& d, struct search_params *sp)
{
    URLSet s = _left->evaluate(d, sp);
    s |= _right->evaluate(d, sp);
    if ( s.length() > sp->max_internal ) {
	cout.form(InternalExceeded, sp->max_internal);
	exit(1);
    }
    return s;
}

inline int UnionSetNode::isNot() 
{ 
    return 0; 
}

inline ostream& UnionSetNode::print(ostream& os)
{ 
    return os << "(" << *_left << " + " << *_right << ")"; 
}



inline CutSetNode::CutSetNode(SetNode* left, SetNode* right)
{
    _left = left; 
    _right = right;
}

inline URLSet CutSetNode::evaluate(Disk& d, struct search_params *sp)
{
    URLSet s = _left->evaluate(d, sp);
    s &= _right->evaluate(d, sp);
    return s;
}

inline int CutSetNode::isNot()
{ 
    return 0;
}

inline ostream& CutSetNode::print(ostream& os)
{ 
    return os << "(" << *_left << " * " << *_right << ")"; 
}


inline DiffSetNode::DiffSetNode(SetNode* left, SetNode* right)
{
    _left = left; 
    _right = right;
}

inline URLSet DiffSetNode::evaluate(Disk& d, struct search_params * sp)
{
    URLSet s = _left->evaluate(d, sp);
    s -= _right->evaluate(d, sp);
    return s;
}

inline int DiffSetNode::isNot() 
{ 
    return 0; 
}

inline ostream& DiffSetNode::print(ostream& os)
{ 
    return os << "(" << *_left << " - " << *_right << ")"; 
}

inline TextSetNode::TextSetNode(unsigned char *str)
{
    _str = str;
}

inline URLSet TextSetNode::evaluate(Disk& d, struct search_params * sp)
{
#ifdef DEBUG
    cerr << "Evaluating \"" << _str << "\"" << endl;
#endif
    URLSet *result = d.Search(_str, sp);

    if ( !result ) {
	cout.form(BadIndex, d.Name());
	exit(1);
    }

    return *result;
}

inline int TextSetNode::isNot() 
{ 	
    return 0; 
}

inline ostream& TextSetNode::print(ostream& os)
{
    return os << '"' << _str << '"';
}

#endif /* SETNODE_H */
